import processing.core.*; 
import processing.xml.*; 

import java.applet.*; 
import java.awt.*; 
import java.awt.image.*; 
import java.awt.event.*; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class sample extends PApplet {

PImage mapImage;
Table locationTable;
Table dataTable;
int rows;

float dmin = MAX_FLOAT;
float dmax = MIN_FLOAT;

Integrator[] interpolators;

public void setup() {
  
  PFont font = loadFont("Verdana-12.vlw");
  textFont(font);
  
  size(640,400);
  mapImage = loadImage("map.png");
  locationTable = new Table("locations.tsv");
  rows = locationTable.getRowCount();

  dataTable = new Table("random.tsv");

  interpolators = new Integrator[rows];
  for(int row=0;row<rows;row++) {
    float value = dataTable.getFloat(row,1);
    interpolators[row]= new Integrator(0,0.9f,0.1f);
    interpolators[row].target(value);
  }
  frameRate(30);
}

public void draw() {
  background(255);
  image(mapImage,0,0);
  
  smooth();
  fill(192,0,0);
  noStroke();
 
  for(int row=0;row<rows; row++) {
    interpolators[row].update();
  }
 
  for(int row=0; row<rows; row++) {
    String abbrev = dataTable.getRowName(row);
    float x = locationTable.getFloat(abbrev,1);
    float y = locationTable.getFloat(abbrev,2);
    drawData(x,y,abbrev);
  }
}

public void drawData(float x, float y, String abbrev) {
  int row = dataTable.getRowIndex(abbrev);
  float value = interpolators[row].value;
  float radius;
  if(value > 0) {
    radius = map(value,0,10,1.5f,15);
    fill(0xff333366);
  } else {
    radius = map(value,0,-10,1.5f,15);
    fill(0xffec5166);
  }
  ellipseMode(RADIUS);
  ellipse(x,y,radius,radius);
  if(dist(x,y,mouseX,mouseY) < radius+2) {
   fill(0);
   textAlign(CENTER); 
   text(nfp(value,0,2) + " (" + abbrev + ") ",x,y-radius-4);
  }
  
}

public void keyPressed() {
  if(key == ' ') {
    updateTable();
  }
}

public void updateTable() {
 for(int row=0;row<rows;row++) {
  float nv = random(-10,10);
    interpolators[row].target(nv);
  }
}
class Integrator {

  final float DAMPING = 0.5f;
  final float ATTRACTION = 0.2f;

  float value;
  float vel;
  float accel;
  float force;
  float mass = 1;

  float damping = DAMPING;
  float attraction = ATTRACTION;
  boolean targeting;
  float target;


  Integrator() { }


  Integrator(float value) {
    this.value = value;
  }


  Integrator(float value, float damping, float attraction) {
    this.value = value;
    this.damping = damping;
    this.attraction = attraction;
  }


  public void set(float v) {
    value = v;
  }


  public void update() {
    if (targeting) {
      force += attraction * (target - value);      
    }

    accel = force / mass;
    vel = (vel + accel) * damping;
    value += vel;

    force = 0;
  }


  public void target(float t) {
    targeting = true;
    target = t;
  }


  public void noTarget() {
    targeting = false;
  }
}
class Table {
  String[][] data;
  int rowCount;
  
  
  Table() {
    data = new String[10][10];
  }

  
  Table(String filename) {
    String[] rows = loadStrings(filename);
    data = new String[rows.length][];
    
    for (int i = 0; i < rows.length; i++) {
      if (trim(rows[i]).length() == 0) {
        continue; // skip empty rows
      }
      if (rows[i].startsWith("#")) {
        continue;  // skip comment lines
      }
      
      // split the row on the tabs
      String[] pieces = split(rows[i], TAB);
      // copy to the table array
      data[rowCount] = pieces;
      rowCount++;
      
      // this could be done in one fell swoop via:
      //data[rowCount++] = split(rows[i], TAB);
    }
    // resize the 'data' array as necessary
    data = (String[][]) subset(data, 0, rowCount);
  }


  public int getRowCount() {
    return rowCount;
  }
  
  
  // find a row by its name, returns -1 if no row found
  public int getRowIndex(String name) {
    for (int i = 0; i < rowCount; i++) {
      if (data[i][0].equals(name)) {
        return i;
      }
    }
    println("No row named '" + name + "' was found");
    return -1;
  }
  
  
  public String getRowName(int row) {
    return getString(row, 0);
  }


  public String getString(int rowIndex, int column) {
    return data[rowIndex][column];
  }

  
  public String getString(String rowName, int column) {
    return getString(getRowIndex(rowName), column);
  }

  
  public int getInt(String rowName, int column) {
    return parseInt(getString(rowName, column));
  }

  
  public int getInt(int rowIndex, int column) {
    return parseInt(getString(rowIndex, column));
  }

  
  public float getFloat(String rowName, int column) {
    return parseFloat(getString(rowName, column));
  }

  
  public float getFloat(int rowIndex, int column) {
    return parseFloat(getString(rowIndex, column));
  }
  
  
  public void setRowName(int row, String what) {
    data[row][0] = what;
  }


  public void setString(int rowIndex, int column, String what) {
    data[rowIndex][column] = what;
  }

  
  public void setString(String rowName, int column, String what) {
    int rowIndex = getRowIndex(rowName);
    data[rowIndex][column] = what;
  }

  
  public void setInt(int rowIndex, int column, int what) {
    data[rowIndex][column] = str(what);
  }

  
  public void setInt(String rowName, int column, int what) {
    int rowIndex = getRowIndex(rowName);
    data[rowIndex][column] = str(what);
  }

  
  public void setFloat(int rowIndex, int column, float what) {
    data[rowIndex][column] = str(what);
  }


  public void setFloat(String rowName, int column, float what) {
    int rowIndex = getRowIndex(rowName);
    data[rowIndex][column] = str(what);
  }
  
  
  // Write this table as a TSV file
  public void write(PrintWriter writer) {
    for (int i = 0; i < rowCount; i++) {
      for (int j = 0; j < data[i].length; j++) {
        if (j != 0) {
          writer.print(TAB);
        }
        if (data[i][j] != null) {
          writer.print(data[i][j]);
        }
      }
      writer.println();
    }
    writer.flush();
  }
}

  static public void main(String args[]) {
    PApplet.main(new String[] { "--present", "--bgcolor=#666666", "--stop-color=#cccccc", "sample" });
  }
}
