PImage mapImage;
Table locationTable;
Table dataTable;
int rows;

float dmin = MAX_FLOAT;
float dmax = MIN_FLOAT;

Integrator[] interpolators;

void setup() {
  
  PFont font = loadFont("Verdana-12.vlw");
  textFont(font);
  
  size(640,400);
  mapImage = loadImage("map.png");
  locationTable = new Table("locations.tsv");
  rows = locationTable.getRowCount();

  dataTable = new Table("random.tsv");

  interpolators = new Integrator[rows];
  for(int row=0;row<rows;row++) {
    float value = dataTable.getFloat(row,1);
    interpolators[row]= new Integrator(0,0.9,0.1);
    interpolators[row].target(value);
  }
  frameRate(30);
}

void draw() {
  background(255);
  image(mapImage,0,0);
  
  smooth();
  fill(192,0,0);
  noStroke();
 
  for(int row=0;row<rows; row++) {
    interpolators[row].update();
  }
 
  for(int row=0; row<rows; row++) {
    String abbrev = dataTable.getRowName(row);
    float x = locationTable.getFloat(abbrev,1);
    float y = locationTable.getFloat(abbrev,2);
    drawData(x,y,abbrev);
  }
}

void drawData(float x, float y, String abbrev) {
  int row = dataTable.getRowIndex(abbrev);
  float value = interpolators[row].value;
  float radius;
  if(value > 0) {
    radius = map(value,0,10,1.5,15);
    fill(#333366);
  } else {
    radius = map(value,0,-10,1.5,15);
    fill(#ec5166);
  }
  ellipseMode(RADIUS);
  ellipse(x,y,radius,radius);
  if(dist(x,y,mouseX,mouseY) < radius+2) {
   fill(0);
   textAlign(CENTER); 
   text(nfp(value,0,2) + " (" + abbrev + ") ",x,y-radius-4);
  }
  
}

void keyPressed() {
  if(key == ' ') {
    updateTable();
  }
}

void updateTable() {
 for(int row=0;row<rows;row++) {
  float nv = random(-10,10);
    interpolators[row].target(nv);
  }
}
